# -*- coding: utf-8 -*-
import re


class Celda(object):

    """Representa una celda de una hoja de calculo.

    Este objeto NO hace parte ni hereda de COM, simplemente ofrece unas
    funciones que facilitan la manipulacion de celdas como cadena de
    texto sin depender de la api especifica de la hoja de datos.

    """

    def __init__(self, celda):
        """Inicializa la variable.

        Args:

            celda -- str en la forma de B12, BG45, etc
        """
        self._celda = None
        self._fila = None
        self._columna = None
        self.celda_set(celda)

    def celda(self):
        """Obtener la celda en notacion de LetrasNumeros."""
        return self._celda

    def celda_set(self, celda):
        """Establece la celda"""
        regex = re.compile(r'[1-9]+')
        match = regex.search(celda)
        if match is None:
            raise StandarError('Celda mal formada %s' % celda)
        columna = celda[0:match.start()]
        columna = self.letra_to_indice(columna)
        fila = int(celda[match.start():])
        self._celda = celda
        self._fila = fila
        self._columna = columna

    def columna(self):
        """Devuelve la columna."""
        return self.indice_a_letra(self._columna)

    def columna_int(self):
        """Devuelve la columna."""
        return self._columna

    def fila(self):
        """Devuelve la fila."""
        return self._fila

    def suma(self, n_columna, n_fila):
        """Suma cierto numeros de columnas y filas a la celda."""
        columna = self._columna + n_columna
        fila = self._fila + n_fila
        nueva_celda = '%s%s' % (self.indice_a_letra(columna), fila)
        return Celda(nueva_celda)

    def indice_a_letra(self, x):
        """Pasa un numero de columna a letras.

        Args:
            x -- Numero entero de la celda.

        Ret:
            Las letras que indican la columna.

        En Excel cuando se va a referenciar a una celda, las columnas se
        nombran con letras, ej: A1 es la columna 1 fila 1. Para efectos
        de cálculos se necesitan las referencias a las colunas como numeros
        enteros.

        Ej:
            >>> indice_a_letter(0)
            'A'
            >>> indice_a_letter(1)
            'B'
            >>> indice_a_letter(26)
            'AA'

        """
        span = ord('Z') - ord('A') + 1
        div = x / span
        letras = ''
        if div > 0:
            letras = letras + self.indice_a_letra(div - 1)
        mod = chr((x % span) + ord('A'))
        letras = letras + mod
        return letras

    def letra_to_indice(self, letras):
        """Pasa una letra de columna a numero.

        Args:
            letras -- Cadena de texto con el indice de la columna.

        Ret:
            Numero entero con el indice.

        >>> letra_to_indice('A')
        0
        >>> letra_to_indice('B')
        1
        >>> letra_to_indice('AA')
        26

        """
        indice = 1
        suma = 0
        for letra in letras:
            suma += (ord(letra) - ord('A') + 1) * (26 ** (len(letras) - indice))
            indice += 1
        return suma - 1
