# -*- coding: utf-8 -*-
import os
import datetime
from tabla import Tabla

class Consulta():

    """Una consulta con informacion adicional para escribirla.

    Contiene informacion sobre una consulta SQL y como se debe escribir
    en el documento.

    """

    def __init__(self, nombre,  query, parametros):
        """Inicializa el objeto

        Args:
            nombre -- Nombre de la consulta.
            query -- Texto con la consulta en SQL
            parametros -- Variables que necesita la consulta

        """
        self.query = query
        self.parametros = parametros
        self.nombre = nombre
        self._tablas = []
        self._resultado = []
        self._resultados = []
        self._ambiente = {}
        self._repetir_con = None

    def tabla_agregar(self, tabla):
        """Agrega un punto de insercion de datos."""
        self._tablas.append(tabla)

    def tablas(self):
        """Obtener las tablas de la consulta."""
        return self._tablas

    def ambiente_set(self, ambiente):
        """Establece al ambiente para la ejecucion de la consulta."""
        self._ambiente = ambiente

    def correr(self, conexion):
        """Ejecuta la consulta.

        Ejecuta la consulta y reemplaza los parametros necesarios

        """
        to_eval = "'%s'" % self.query.replace('\n', ' ')
        if not self.parametros is None:
            to_eval = to_eval + ' % ' + self.parametros

        query = eval(to_eval, {}, self._ambiente)
        self._resultado = conexion.ejecutar(query)

    def data_get(self, tabla, ambiente={}):
        """Devuelve la informacion a escribir en el punto."""
        return tabla.data_get(self._resultado, ambiente)

    def resultado(self):
        """Devuelve los datos de la consulta."""
        return self._resultado

    def repetir_con(self):
        """Obtener la variable sobre la que se debe iterar."""
        return self._repetir_con

    def repetir_con_set(self, repetir_con):
        """Establece la variable para repetir la consulta."""
        self._repetir_con = repetir_con
