#!/usr/bin/python
import os
if not os.name == 'nt' and __name__ != 'exportador_com.main':
    raise StandardError('Este plugin solo funciona en Windows.')

import datetime

import gtk

from plugins import base
from gP7.wdg_auto_wrap_label import AutoWrapLabel
from yaml_parser import YAMLParser as Parser


class Main(base.Plugin):

    """Maneja los puntos de entrada del plugin."""

    def __init__(self, interface):
        """Inicializa el objeto.

        Args:
            interface -- Interface con el programa

        La interface es el objeto que le da acceso a las funciones del
        programa, adicionar menues, hacer consultas, etc.

        """
        self._interface = interface
        base.Plugin.__init__(self, 'Exportador a Excel')
        interface.agregar_menu('Exportar formatos a Excel', self.explorador)

    def explorador(self, widget):
        """Permite escoger entre los reportes disponibles."""
        a = _Exportador(self._interface)
        a.listar()


class _Exportador():

    """Ventana para escoge el formato."""

    def __init__(self, interface):
        self._interface = interface

        # Controles graficos
        self._window = gtk.Window()
        self._vb_principal = gtk.VBox()
        self._mdl_informes = gtk.ListStore(str, str, str, str, str, str)
        self._sw_listado = gtk.ScrolledWindow()
        self._tv_listado = gtk.TreeView()
        self._tvc_nombre = gtk.TreeViewColumn('Informes')
        self._crt_nombre = gtk.CellRendererText()
        self._frm_informacion = gtk.Frame('Informacion')
        self._tbl_informacion = gtk.Table(6, 2)
        self._lbl_titulo_autor = gtk.Label('Autor:')
        self._lbl_autor = gtk.Label('')
        self._lbl_titulo_email = gtk.Label('Email:')
        self._lbl_email = gtk.Label('')
        self._lbl_titulo_edicion = gtk.Label('Edicion')
        self._lbl_edicion = gtk.Label('')
        self._lbl_titulo_descripcion = gtk.Label('Descripcion:')
        self._lbl_descripcion = AutoWrapLabel('')
        self._hb_botones = gtk.HBox()
        self._cmd_ok = gtk.Button(stock=gtk.STOCK_OK)
        self._cmd_salir = gtk.Button(stock=gtk.STOCK_CLOSE)

        # Empaquetar
        self._window.add(self._vb_principal)
        self._vb_principal.pack_start(self._sw_listado)
        self._sw_listado.add(self._tv_listado)
        self._tv_listado.append_column(self._tvc_nombre)
        self._tvc_nombre.pack_start(self._crt_nombre)
        self._vb_principal.pack_start(self._frm_informacion, expand=False)
        self._vb_principal.pack_start(self._hb_botones, expand=False)
        self._frm_informacion.add(self._tbl_informacion)
        self._tbl_informacion.attach(self._lbl_titulo_autor, 0, 1, 0, 1, xoptions=gtk.SHRINK)
        self._tbl_informacion.attach(self._lbl_autor, 1, 2, 0, 1)
        self._tbl_informacion.attach(self._lbl_titulo_email, 0, 1, 1, 2, xoptions=gtk.SHRINK)
        self._tbl_informacion.attach(self._lbl_email, 1, 2, 1, 2)
        self._tbl_informacion.attach(self._lbl_titulo_edicion, 0, 1, 2, 3, xoptions=gtk.SHRINK)
        self._tbl_informacion.attach(self._lbl_edicion, 1, 2, 2, 3)
        self._tbl_informacion.attach(self._lbl_titulo_descripcion, 0, 1, 3, 4, xoptions=gtk.SHRINK)
        self._tbl_informacion.attach(self._lbl_descripcion, 1, 2, 3, 4)
        self._hb_botones.pack_start(self._cmd_ok)
        self._hb_botones.pack_start(self._cmd_salir)

        # Propiedades
        self._window.set_title('Informes')
        self._window.set_icon_from_file("iconos/p_16x16.png")
        self._sw_listado.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
        self._tv_listado.set_model(self._mdl_informes)
        self._tvc_nombre.add_attribute(self._crt_nombre, 'text', 1)
        self._tbl_informacion.set_col_spacings(10)
        self._window.set_border_width(5)
        self._window.set_size_request(300, 300)
        self._lbl_titulo_autor.set_alignment(xalign=0, yalign=0.5)
        self._lbl_titulo_email.set_alignment(xalign=0, yalign=0.5)
        self._lbl_titulo_edicion.set_alignment(xalign=0, yalign=0.5)
        self._lbl_titulo_descripcion.set_alignment(xalign=0, yalign=0)
        self._lbl_autor.set_alignment(xalign=0, yalign=0.5)
        self._lbl_email.set_alignment(xalign=0, yalign=0.5)
        self._lbl_edicion.set_alignment(xalign=0, yalign=0.5)
        self._lbl_descripcion.set_alignment(xalign=0, yalign=0.5)

        # Conectar funciones de controles
        self._tv_listado.connect('cursor-changed', self._seleccionado)
        self._cmd_ok.connect('clicked', self._exportar)
        self._cmd_salir.connect('clicked', self._salir)

        # Mostrar
        self._window.show_all()

    def listar(self):
        """Encuentra los reportes instalados.

        Lista los reportes que se encuentran instalados en el plugin
        para que puedan ser seleccionado el necesario en la interfaz.
        """
        (carpeta, nombre_py) = os.path.split(__file__)
        b = Parser()
        for nombre in os.listdir(carpeta):
            (archivo, extension) = os.path.splitext(nombre)
            if extension == '.txt':
                a = b.parse(os.path.join(carpeta, nombre), True)
                self._mdl_informes.append((nombre,
                                           a._nombre,
                                           a._autor,
                                           a._email,
                                           a._edicion,
                                           a._descripcion))

    def _seleccionado(self, widget):
        """Muestra la informacion del informe seleccionado."""
        iterador = self._tv_listado.get_selection().get_selected()[1]
        if not iterador:
            return
        self._lbl_autor.set_text(self._mdl_informes.get_value(iterador, 2))
        self._lbl_email.set_text(self._mdl_informes.get_value(iterador, 3))
        self._lbl_edicion.set_text(self._mdl_informes.get_value(iterador, 4))
        self._lbl_descripcion.set_text(self._mdl_informes.get_value(iterador, 5))

    def _exportar(self, widget):
        """Exporta el archivo seleccionado."""
        iterador = self._tv_listado.get_selection().get_selected()[1]
        if not iterador:
            return

        archivo = self._mdl_informes.get_value(iterador, 0)
        (prefijo, nombre) = os.path.split(__file__)
        archivo_control = os.path.join(prefijo, archivo)
        parser = Parser()
        formato = parser.parse(archivo_control)

        argumentos = {'hoy': datetime.date.today().isoformat(),
                      'dir_fotos': self._interface.configuracion().dir_fotos}
        for variable in formato.datos_usuario():
            mensaje = formato.datos_usuario()[variable]
            argumentos[variable] = self._interface.input_usuario(mensaje)

        formato.argumentos_set(argumentos)
        formato.exportar(self._interface.proyecto_get())


    def _salir(self, widget):
        """Cierra el dialogo."""
        self._window.destroy()


