from celda import Celda

class Tabla(object):

    """Representa una  tabla de datos.

    Esta tabla tiene la informacion necesaria para plasmar los datos
    en un hoja de calculo.
    """

    def __init__(self, nombre):
        """Inicializa el objeto

        Args:
            celda -- Celda. ej: 'E13'

        """
        self._nombre = nombre
        self._celda = None
        self._hoja = None
        self._hoja_base = ""
        self._transformada = []
        self._expandir_filas = False
        self._expandir_col = False
        self._tipo = 'data'
        self._guardar_en = None

    def data_get(self, datos, ambiente):
        """Devuelve los datos transformados."""
        if len(datos) == 0:
            return datos

        for funcion in self._transformada.pila():
            parametros = []
            for parametro in funcion[1]:
                parametros.append(eval(parametro, {}, ambiente))

            datos = funcion[0](datos, *parametros)

        if not self.guardar_en() is None:
            ambiente[self.guardar_en()] = datos

        return datos

    def nombre(self):
        """Obtener el nombre de la tabla."""
        return self._nombre

    def celda(self):
        """Devuelve la celda."""
        return self._celda

    def celda_set(self, celda):
        """Establece la celda"""
        self._celda = Celda(celda)

    def hoja(self):
        """Obtener la hoja de la tabla"""
        return self._hoja

    def hoja_set(self, hoja):
        """Establecer la hoja de la tabla."""
        self._hoja = hoja

    def hoja_base(self):
        """Obtener hoja base para copiar en caso de que la hoja no exista."""
        return self._hoja_base

    def hoja_base_set(self, hoja_base):
        """Obtener la hoja base a copiar de la tabla."""
        self._hoja_base = hoja_base

    def transformada(self):
        """Obtener la transformada de la tabla."""
        return self._transformada

    def transformada_set(self, transformada):
        """Establece la transformada de la tabla."""
        self._transformada = transformada

    def expandir_filas(self):
        """Obtener si se deben expandir o no las filas."""
        return self._expandir_filas

    def expandir_filas_set(self, expandir_filas):
        """Establece si se debe expandir la tabla en las filas."""
        self._expandir_filas = expandir_filas

    def expandir_columnas(self):
        """Obtener si se deben expandir o no las columnas."""
        return self._expandir_col

    def expandir_columnas_set(self, expandir_columnas):
        """Establece si se debe expandir la tabla en las columnas."""
        self._expandir_col = expandir_columnas

    def tipo(self):
        """Obtener el tipo de tabla"""
        return self._tipo

    def tipo_set(self, tipo):
        """Establecer el tipo de tabla."""
        if tipo != 'data':
            raise StandardError('Tipo de tabla incorrecto: %s' % tipo)
        self._tipo = tipo

    def guardar_en(self):
        """Obtener el nombre de la variable donde guardar el resultado."""
        return self._guardar_en

    def guardar_en_set(self, guardar_en):
        """Establece el nombre de la variable donde guardar el resultado."""
        self._guardar_en = guardar_en
