# -*- coding: utf-8 -*-
import os

from com_writer import COMWriter


class XlsFormato(object):

    """Lee un archivo de control y exporta un formato usando COM.

    Esta clase escribe datos de la base de datos en archivos de excel usando
    COM. Se necesitan dos archivos para poder hacer esto, uno es la plantilla
    de excelvy otro es el archivo de control. El archivo de control especifica
    que plantilla se debe usar.

    El archivo de plantila es un archivo de excel que contiene el formato,
    este archivo contiene los colores, bordes, estilos de letra y en general
    todo lo que se refiere al formato de los datos dentro de excel.

    El archivo de control especifica que datos se van a escribir en la
    plantilla, en que celdas de la plantilla se deben escribir los datos y
    que informacion se necesita del usuario.

    """

    def __init__(self):
        """Inicializa el objeto."""
        self._conexion = None
        self._consultas = []
        self._imagenes = []
        self._nombre = ""
        self._descripcion = ""
        self._version = 0
        self._autor = ""
        self._email = ""
        self._edicion = ""
        self._plantilla = ""
        self._macros = False
        self._archivo_salida = ""
        self._datos_usuario = []
        self._ambiente = {}

    def documento(self, archivo):
        """Devuel el archivo donde escribir las tablas."""
        directorio = os.path.split(os.path.abspath(__file__))[0]
        if archivo is None:
            archivo = self._archivo_salida.format(**self._ambiente)

        ruta_plantilla = os.path.join(directorio, self._plantilla)
        documento = COMWriter(ruta_plantilla, archivo)
        return documento

    def ambientes_generar(self, consulta):
        """Genera el ambiente para una consulta.

        Algunas consultas se deben generar para ciero conjuntos
        de parametros.
        """
        ambientes = []
        if consulta.repetir_con() is None:
            return [self._ambiente]

        else:
            valores = self._ambiente[consulta.repetir_con()]
            for dato in valores:
                nuevo_ambiente = dict(self._ambiente)
                # Los datos estan es tuples, son filas eso se usa [0]
                nuevo_ambiente[consulta.repetir_con()] = dato[0]
                ambientes.append(nuevo_ambiente)

            return ambientes

    def exportar(self, proyecto, archivo=None):
        """Exporta el documento a un archivo usando COM."""
        self._conexion = proyecto.conexion
        documento = self.documento(archivo)

        n_consultas = len(self._consultas)
        for (i, consulta) in enumerate(self._consultas):
            print "%.f%%: %s" % (float(i) / n_consultas * 100, consulta.nombre)
            ambientes = self.ambientes_generar(consulta)
            for ambiente in ambientes:
                consulta.ambiente_set(ambiente)
                consulta.correr(self._conexion)
                for tabla in consulta.tablas():
                    datos = consulta.data_get(tabla, ambiente)
                    if not tabla.celda() is None:
                        self.tabla_escribir(tabla, datos, documento, ambiente)

        for imagen in self._imagenes:
            imagen.ruta = eval(imagen.ruta, {}, self._ambiente)
            ruta_completa = os.path.join(self._ambiente['dir_fotos'],
                                         imagen.ruta)
            imagen.ruta = os.path.abspath(ruta_completa)
            self.imagen_escribir(documento, imagen)

        documento.guardar(self._macros)

    def imagen_escribir(self, documento, imagen):
        """Escribe una imagen en la hoja."""
        documento.insertar_foto(imagen.celda_inicial, imagen.celda_final,
                                imagen.ruta)

    def tabla_escribir(self, tabla, datos, documento, ambiente):
        """Escribir una tabla"""
        if len(datos) == 0:
            return
        numero_filas = len(datos)
        numero_columnas = len(datos[0])
        expandir_filas = tabla.expandir_filas()
        expandir_columnas = tabla.expandir_columnas()
        if expandir_filas and (expandir_filas[0] < numero_filas):
            cantidad = numero_filas - expandir_filas[0]
            documento.expandir_filas(tabla.celda(), cantidad)
            expandir_filas[0] = numero_filas

        if expandir_columnas and (expandir_columnas[0] < numero_columnas):
            cantidad = numero_columnas - expandir_columnas[0]
            documento.expandir_columnas(tabla.celda(), cantidad)
            expandir_columnas[0] = numero_columnas

        if tabla.tipo() == 'data':
            hoja = self._hoja(tabla.hoja(), ambiente)
            documento.escribir(tabla, datos, hoja, tabla.hoja_base())

    def consulta_agregar(self, consulta):
        """Agrega una consulta al documento."""
        self._consultas.append(consulta)

    def imagen_agregar(self, imagen):
        """Agregar imagen."""
        self._imagenes.append(imagen)

    def argumentos_set(self, argumentos):
        """Estable las variables del ambiente.

        Estas variables se utilizan para la evaluacion de expresiones del
        archivo de control. Como el nombre del archivo de salida o el nombre
        de las hojas.

        """
        self._ambiente = argumentos

    def _hoja(self, hoja, ambiente):
        """Devuelve la hoja de excel."""
        if hoja is None:
            return None
        return hoja.format(**ambiente)

    def nombre_set(self, nombre):
        """Establece el nombre del documento."""
        self._nombre = nombre

    def nombre(self):
        """Obtener el nombre del archivo."""
        return self._nombre

    def descripcion(self):
        """Obtener la descripcion del documento."""
        return self._descripcion

    def descripcion_set(self, descripcion):
        """Estabelecer la descripcion del documento."""
        self._descripcion = descripcion

    def version(self):
        """Obtener la version necesaria de exportador_com."""
        return self._version

    def version_set(self, version):
        """Establecer la version necesaria de exportador_com."""
        self._version = version

    def autor(self):
        """Obtener el nombre del autor del documento."""
        return self._autor

    def autor_set(self, autor):
        """Establecer el nombre del autor del documento."""
        self._autor = autor

    def email(self):
        """Obtenre el email del autor del documento."""
        return self._email

    def email_set(self, email):
        self._email = email

    def edicion(self):
        """Obtener la fecha de edicion del documento."""
        return self._edicion

    def edicion_set(self, edicion):
        """Establecer la fecha de edicion del documento."""
        self._edicion = edicion

    def plantilla(self):
        """Obtener la plantilla del documento"""
        return self._plantilla

    def plantilla_set(self, plantilla):
        """Establecer la plantilla del documento."""
        self._plantilla = plantilla

    def macros(self):
        """Obtener si la plantila tiene macros."""
        return self._macros

    def macros_set(self, macros):
        """Establece si la plantilla tiene macros."""
        self._macros = macros

    def archivo_salida(self):
        """Obtener el archivo de salida."""
        return self._archivo_salida

    def archivo_salida_set(self, archivo_salida):
        """Establecer la ruta del archivo de salida."""
        self._archivo_salida = archivo_salida

    def datos_usuario(self):
        """Obtener los datos del usuario."""
        return self._datos_usuario

    def datos_usuario_set(self, datos):
        self._datos_usuario = datos
